package me.mrletsplay.ollama.api;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

import me.mrletsplay.mrcore.json.JSONArray;
import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.ollama.api.ChatMessage.Role;

public class Chat {

	public static final int DEFAULT_HISTORY_SIZE = -1;

	private OllamaAPI api;
	private String model;
	private int historySize;

	private List<ChatMessage> messages;

	public Chat(OllamaAPI api, String model) {
		this.api = api;
		this.model = model;
		this.historySize = DEFAULT_HISTORY_SIZE;
		this.messages = new ArrayList<>();
	}

	/**
	 * -1 == unlimited
	 * @param historySize
	 */
	public void setHistorySize(int historySize) {
		this.historySize = historySize;
	}

	public int getHistorySize() {
		return historySize;
	}

	public void sendSystem(String message) {
		this.messages.add(new ChatMessage(Role.SYSTEM, message));
	}

	public void sendAssistant(String message) {
		this.messages.add(new ChatMessage(Role.ASSISTANT, message));
	}

	public void sendUser(String message) {
		this.messages.add(new ChatMessage(Role.USER, message));
	}

	public Stream<String> doCompletion() throws IOException, InterruptedException {
		StringBuffer message = new StringBuffer();
		return api.prompt(this)
			.map(o -> {
				String content = o.getJSONObject("message").getString("content");
				message.append(content);

				if(o.getBoolean("done")) {
					sendAssistant(message.toString());
				}

				return content;
			});
	}

	public JSONObject toJSON() {
		JSONObject obj = new JSONObject();
		obj.put("model", model);

		JSONArray messages = new JSONArray();
		// TODO: keep system messages
		this.messages.stream().skip(historySize == -1 ? 0 : Math.max(0, messages.size() - historySize)).forEach(m -> messages.add(m.toJSON()));
		obj.put("messages", messages);

//		JSONObject options = new JSONObject();
//		options.put("num_thread", 16);
//		obj.put("options", options);
		return obj;
	}

}
